//
// JTypist v3.1 - Java typing tutor program for UNIX systems
// Copyright (C) 2000  Simon Baldwin (simonb@sco.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//
// ControlGUI class - links lesson data and other frames together
//
import java.awt.Frame;
import java.awt.Font;
import java.awt.Panel;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Choice;
import java.awt.Label;
import java.awt.List;
import java.awt.Menu;
import java.awt.MenuBar;
import java.awt.MenuItem;
import java.awt.MenuShortcut;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;
import java.awt.event.WindowListener;
import java.awt.event.WindowEvent;
import java.util.Vector;
import java.util.Hashtable;
import java.util.ResourceBundle;
import java.text.MessageFormat;

/**
 * Creates a the Typist tutor's control window.
 */
public class ControlGUI extends Frame
		implements ItemListener, ActionListener, WindowListener {
	private boolean inAnApplet = true;

	// message and colour catalog information
	private static final ResourceBundle	messages	=
				ResourceBundle.getBundle( "MessageResource" );
	private static final ResourceBundle	colours		=
				ResourceBundle.getBundle( "ColorResource" );

	// other frames that we have to control
	private TutorialGUI	tutorial;//	= null;
	private HelpGUI		help;//		= null;
	private AboutGUI	about;//	= null;
	private ExerciseGUI	exercise;//	= null;
	private StatisticsGUI	statistics;//	= null;
	private KeyboardGUI	keyboard;//	= null;
	private PopupGUI	popup;//	= null;

	// lesson library data; series id's offered, and the series
	// instances themselved, hashed by id
	private Vector		offered_series_ids	= new Vector();
	private Hashtable	loaded_series		= new Hashtable();

	// information about the currently selected series
	private LessonSeries	current_series;//	= null;
	private int		current_series_index	= -1;

	// menubar GUI components
	private MenuBar		menu_bar;
	private Menu		file_menu, window_menu, help_menu;
	private MenuItem	file_exit, file_prev, file_next;
	private MenuItem	window_help, window_tutorial, window_exercise,
				window_statistics, window_keyboard;
	private MenuItem	help_about, help_on_typist;

	// GUI components
	private ImageButton	prev_button, next_button;
	private Tooltip		prev_tooltip, next_tooltip;
	private Choice		series_choice;
	private Tooltip		series_choice_tooltip;
	private Label		series_label;
	private List		lesson_list;
	private Label		status_label;


	/**
	 * Initializes the whole GUI set.
	 */
	public ControlGUI( boolean inAnApplet ) {
		this.addWindowListener( this );
		this.inAnApplet = inAnApplet;

		// create GUI components
		setTitle( messages.getString( "ControlGUI.title" ));
		setLayout( new BorderLayout() );
		setForeground( Colour.find_colour(
			colours.getString( "ControlGUI.foreground" )));
		setBackground( Colour.find_colour(
			colours.getString( "ControlGUI.background" )));
		setFont( Font.decode(
			colours.getString( "ControlGUI.font" )));

		// create the menu bar stuff
		menu_bar = new MenuBar();
		setMenuBar( menu_bar );
		// File menu
		file_menu = new Menu( messages.getString( "ControlGUI.file" ));
		menu_bar.add( file_menu );
		file_prev = new MenuItem(
			messages.getString( "ControlGUI.file_prev" ),
			new MenuShortcut(
				messages.getString(
				"ControlGUI.file_prev_sc" ).charAt(0) ));
		file_prev.addActionListener( this );
		file_menu.add( file_prev );
		file_next = new MenuItem(
			messages.getString( "ControlGUI.file_next" ),
			new MenuShortcut(
				messages.getString(
				"ControlGUI.file_next_sc" ).charAt(0) ));
		file_next.addActionListener( this );
		file_menu.add( file_next );
		file_menu.add( new MenuItem( "-" ));

		file_exit = new MenuItem(
			messages.getString( "ControlGUI.file_exit" ));
		file_exit.addActionListener( this );
		file_menu.add( file_exit );
		// Window menu - do everything except add the menu items
		// to the bar; this we do in the routines that register
		// each sub-GUI with us
		window_menu = new Menu(
			messages.getString( "ControlGUI.window" ));
		menu_bar.add( window_menu );
		window_help = new MenuItem(
			messages.getString( "ControlGUI.window_help" ));
		window_help.addActionListener( this );
		//window_menu.add( window_help );
		window_tutorial = new MenuItem(
			messages.getString( "ControlGUI.window_tutorial" ),
			new MenuShortcut(
				messages.getString(
				"ControlGUI.window_tutorial_sc" ).charAt(0) ));
		window_tutorial.addActionListener( this );
		//window_menu.add( window_tutorial );
		window_exercise = new MenuItem(
			messages.getString( "ControlGUI.window_exercise" ),
			new MenuShortcut(
				messages.getString(
				"ControlGUI.window_exercise_sc" ).charAt(0) ));
		window_exercise.addActionListener( this );
		//window_menu.add( window_exercise );
		window_statistics = new MenuItem(
			messages.getString( "ControlGUI.window_statistics" ),
			new MenuShortcut(
				messages.getString(
				"ControlGUI.window_statistics_sc" ).charAt(0) ));
		window_statistics.addActionListener( this );
		//window_menu.add( window_statistics );
		window_keyboard = new MenuItem(
			messages.getString( "ControlGUI.window_keyboard" ),
			new MenuShortcut(
				messages.getString(
				"ControlGUI.window_keyboard_sc" ).charAt(0) ));
		window_keyboard.addActionListener( this );
		//window_menu.add( window_keyboard );
		// Help menu
		help_menu = new Menu(
			messages.getString( "ControlGUI.help" ));
		menu_bar.add( help_menu );
		menu_bar.setHelpMenu( help_menu );
		help_about = new MenuItem(
			messages.getString( "ControlGUI.help_about" ),
			new MenuShortcut(
				messages.getString(
				"ControlGUI.help_about_sc" ).charAt(0) ));
		help_about.addActionListener( this );
		//help_menu.add( help_about );
		help_on_typist = new MenuItem(
			messages.getString( "ControlGUI.help_on" ),
			new MenuShortcut(
				messages.getString(
				"ControlGUI.help_on_sc" ).charAt(0) ));
		help_on_typist.addActionListener( this );
		//help_menu.add( help_on_typist );

		// create a toolbar panel
		Panel toolbar = new Panel();
		toolbar.setLayout( new BorderLayout() );
		add( "North", toolbar );

		Panel controls = new Panel();
		controls.setLayout( new FlowLayout());
		toolbar.add( "West", controls );
		prev_button = new ImageButton(
			Xpm.get_image_producer(
				messages.getString( "ControlGUI.prevxpm" )),
			messages.getString( "ControlGUI.prevxpmlabel" ));
		prev_tooltip = new Tooltip( this, prev_button,
			messages.getString( "ControlGUI.prevtooltip" ));
		prev_button.addActionListener( this );
		controls.add( prev_button );
		next_button = new ImageButton(
			Xpm.get_image_producer(
				messages.getString( "ControlGUI.nextxpm" )),
			messages.getString( "ControlGUI.nextxpmlabel" ));
		next_tooltip = new Tooltip( this, next_button,
			messages.getString( "ControlGUI.nexttooltip" ));
		next_button.addActionListener( this );
		controls.add( next_button );
		series_choice = new Choice();
		series_choice_tooltip = new Tooltip( this, series_choice,
			messages.getString( "ControlGUI.seriestooltip" ));
		series_choice.addItemListener( this );
		controls.add( series_choice );

		series_label = new Label();
		toolbar.add( "Center", series_label );

		lesson_list = new List( Integer.parseInt(
				messages.getString( "ControlGUI.listrows" )),
				false );
		lesson_list.addItemListener( this );
		lesson_list.setForeground( Colour.find_colour(
			colours.getString( "ControlGUI.panel_foreground" )));
		lesson_list.setBackground( Colour.find_colour(
			colours.getString( "ControlGUI.panel_background" )));
		lesson_list.setFont( Font.decode(
			colours.getString( "ControlGUI.panel_font" )));
		add( "Center", lesson_list );

		status_label = new Label(
			messages.getString( "ControlGUI.status_none" ),
				Label.RIGHT );
		add( "South", status_label );
	}


	/**
	 * Add a lesson series to those offered.
	 */
	public void add_series( String series_id ) {

		// add this series id into the choice and vector
		String[] fmtarg = { series_id };
		series_choice.add(
			MessageFormat.format(
			messages.getString( "ControlGUI.series" ), fmtarg ));
		offered_series_ids.addElement( series_id );

		// if this is the first series added then automatically
		// select it for use
		if ( offered_series_ids.size() == 1 ) {
			series_choice.select( 0 );
			handle_series_selection();
		}
	}


	/*
	 * Display the given lesson series in our list component.
	 */
	private void display_series( LessonSeries lesson_series ) {

		// show progress
		String[] fmtarg = { lesson_series.get_series_id() };
		status_label.setText(
			MessageFormat.format(
			messages.getString( "ControlGUI.formatting" ),
					fmtarg ));

		// update the lesson list
		lesson_list.setVisible( false );
		if ( lesson_list.getItemCount() > 0 )
			lesson_list.removeAll();
		for ( int index = 0; index < lesson_series.get_length();
				index++ ) {

			// get the lesson title and description
			Lesson lesson	= lesson_series.get( index );
			String title	= lesson.get_title();
			if ( lesson instanceof ExerciseLesson )  {
				String instructions
					= ((ExerciseLesson)lesson).get_instructions();
				if ( ! instructions.equals( "" ) )
					lesson_list.add( title
						+ " - " + instructions, index );
				else
					lesson_list.add( title, index );
			}
			else // ( lesson instanceof TutorialLesson )
				lesson_list.add( title, index );
		}
		lesson_list.setVisible( true );

		// reset the label description
		series_label.setText( lesson_series.get_descr() );
		status_label.setText(
			messages.getString( "ControlGUI.ready" ));
	}


	/*
	 * Set up the sensitivity of the next and prev buttons, depending
	 * on the list display (i.e. is next/prev valid ).
	 */
	private void set_next_prev_sensitivity() {

		// see what, if anything is selected on the list
		int selected = lesson_list.getSelectedIndex();

		// if something is selected, work out where it is in
		// the list, and if at either end disable the required
		// buttons; and do it all with flags to avoid having
		// the buttons flicker
		boolean prev_enable	= false;
		boolean next_enable	= false;
		if ( selected >= 0 ) {
			if ( selected > 0 )
				prev_enable = true;
			if ( selected < lesson_list.getItemCount() - 1 )
				next_enable = true;
		}

		// update the buttons given the flags
		prev_button.setEnabled( prev_enable );
		file_prev.setEnabled( prev_enable );
		next_button.setEnabled( next_enable );
		file_next.setEnabled( next_enable );

		// update the tooltips for the buttons
		if ( ! prev_enable )
			prev_tooltip.setVisible( false );
		if ( ! next_enable )
			next_tooltip.setVisible( false );
	}


	/*
	 * Return a lesson series given just a series id -- load if
	 * required.
	 */
	private LessonSeries get_lesson_series( String series_id ) {

		// try to find this series in the cache
		LessonSeries series =
			(LessonSeries)loaded_series.get( series_id );

		// if the series was not cached, try to load it
		if ( series == null ) {
			String[] fmtarg = { series_id };
			status_label.setText(
				MessageFormat.format(
				messages.getString( "ControlGUI.loading" ),
					fmtarg ));

			// instantiate the lesson series
			series = LessonSeries.create_lessonseries( series_id );

			// if the load worked, cache the result
			if ( series != null )
				loaded_series.put( series_id, series );
		}

		// return the lesson series, or null if failed
		return ( series );
	}


	/*
	 * Update the lessons list based on the currently selected
	 * series.
	 */
	private void handle_series_selection() {

		// get the series selected from the choice
		int	selected_index	= series_choice.getSelectedIndex();

		// if we have no series yet, or if the selected series is
		// not the one we're currently viewing, then do something
		// with the selection; otherwise, ignore it
		if ( current_series_index == -1
				|| selected_index != current_series_index ) {

			// get the series id from the selection
			String series_id =
					(String)offered_series_ids.elementAt(
						selected_index );

			// get the lesson series from the id
			LessonSeries new_series =
					get_lesson_series( series_id );

			// reset the lesson current set and label description,
			// if we (now) have this series
			if ( new_series != null ) {
				// update the series list with this series
				display_series( new_series );

				// reset our notion of the current series
				current_series_index	= selected_index;
				current_series		= new_series;
			}
			else {

				// an error occurred in setting this series, so
				// pop up an error dialog
				if ( popup == null )
					popup = new PopupGUI( inAnApplet,
								this, false );
				String[] fmtarg = { series_id };
				popup.reveal(
				MessageFormat.format(
					messages.getString( "ControlGUI.problem" ),
					fmtarg ));

				// reset the choice to match any current series
				if ( current_series_index != -1 )
					series_choice.select( current_series_index );
				status_label.setText(
					messages.getString( "ControlGUI.ready" ));
			}
		}

		// update button sensitivity, and collect garbage
		set_next_prev_sensitivity();
		garbage_collect();
	}


	/*
	 * Handle the selection of a lesson list item.
	 */
	private void handle_lesson_selection() {

		// if something is selected action it
		int selected = lesson_list.getSelectedIndex();
		if ( selected >= 0 ) {

			// For some reason the List getRows() won't give
			// the right answer about the number of visible
			// rows ???
			// If it did, here would would be the place to
			// jiggle the list to keep the selected item in view

			// now just run the lesson based on lesson type
			Lesson lesson = current_series.get( selected );

			// is this a tutorial?
			if ( lesson instanceof TutorialLesson
					&& tutorial != null )
				tutorial.reveal( (TutorialLesson)lesson );

			// is this a drill or paragraph?
			else if ( lesson instanceof ExerciseLesson
					&& exercise != null )
				exercise.reveal( (ExerciseLesson)lesson );

			// do some housekeeping
			String[] fmtarg = { lesson.get_title() };
			status_label.setText(
				MessageFormat.format(
				messages.getString( "ControlGUI.selected" ),
					fmtarg ));
			set_next_prev_sensitivity();
			garbage_collect();
		}
	}

	/*
	 * Select the previous lesson from the list if available.
	 */
	private void handle_prev_button() {

		// move to previous list item if available
		int selected = lesson_list.getSelectedIndex();
		if ( selected > 0 ) {
			selected--;
			lesson_list.select( selected );
			handle_lesson_selection();
		}
	}


	/*
	 * Select the next lesson from the list if available.
	 */
	private void handle_next_button() {

		// move to next list item if available
		int selected = lesson_list.getSelectedIndex();
		if ( selected < lesson_list.getItemCount() - 1 ) {
			selected++;
			lesson_list.select( selected );
			handle_lesson_selection();
		}
	}


	/*
	 * Handle GUI component actions and events
	 */
	public void actionPerformed( ActionEvent evt ) {
		if ( evt.getSource() instanceof Choice ) {
			handle_series_selection();
		}
		else if ( evt.getSource() instanceof ImageButton ) {
			if ( evt.getSource() == prev_button )
				handle_prev_button();
			else if ( evt.getSource() == next_button )
				handle_next_button();
		}
		else if ( evt.getSource() instanceof MenuItem ) {
			if ( evt.getSource() == file_prev )
				handle_prev_button();
			else if ( evt.getSource() == file_next )
				handle_next_button();
			else if ( evt.getSource() == file_exit ) {
				if ( inAnApplet ) {
					if ( tutorial != null ) tutorial.dispose();
					if ( exercise != null ) exercise.dispose();
					if ( help != null ) help.dispose();
					if ( statistics != null ) statistics.dispose();
					if ( keyboard != null ) keyboard.dispose();
					if ( about != null ) about.dispose();
					dispose();
				}
				else
					System.exit( 0 );
			} else if ( evt.getSource() == window_tutorial ) {
				if ( tutorial != null )
					tutorial.setVisible( true );
			} else if ( evt.getSource() == window_exercise ) {
				if ( exercise != null )
					exercise.setVisible( true );
			} else if ( evt.getSource() == window_help ) {
				if ( help != null )
					help.setVisible( true );
			} else if ( evt.getSource() == window_statistics ) {
				if ( statistics != null )
					statistics.setVisible( true );
			} else if ( evt.getSource() == window_keyboard ) {
				if ( keyboard != null )
					keyboard.setVisible( true );
			} else if ( evt.getSource() == help_about ) {
				if ( about != null )
					about.setVisible( true );
			} else if ( evt.getSource() == help_on_typist ) {
				if ( help != null )
					help.setVisible( true );
			}
		}
	}
	public void itemStateChanged( ItemEvent evt ) {
		if ( evt.getSource() instanceof Choice ) {
			handle_series_selection();
		}
		else if ( evt.getSource( )instanceof List ) {
			handle_lesson_selection();
		}
        }
	public void windowClosing( WindowEvent evt ) {
		if ( inAnApplet ) {
			if ( tutorial != null )	tutorial.dispose();
			if ( exercise != null )	exercise.dispose();
			if ( help != null ) help.dispose();
			if ( statistics != null ) statistics.dispose();
			if ( keyboard != null ) keyboard.dispose();
			if ( about != null ) about.dispose();
			dispose();
		}
		else
			System.exit( 0 );
        }
	public void windowClosed( WindowEvent evt ) {}
	public void windowOpened( WindowEvent evt ) {}
	public void windowIconified( WindowEvent evt ) {}
	public void windowDeiconified( WindowEvent evt ) {}
	public void windowActivated( WindowEvent evt ) {}
	public void windowDeactivated( WindowEvent evt ) {}


	/*
	 * Gather up any garbage that might have accumulated.
	 */
	private void garbage_collect() { Runtime.getRuntime().gc(); }


	/*
	 * Register the other frames we need to control.
	 */
	public void register_tutorial( TutorialGUI tutorial ) {
		this.tutorial = tutorial; garbage_collect();
		window_menu.add( window_tutorial );
	}
	public void register_exercise( ExerciseGUI exercise ) {
		this.exercise = exercise; garbage_collect();
		window_menu.add( window_exercise );
	}
	public void register_help( HelpGUI help ) {
		this.help = help; garbage_collect();
		window_menu.add( window_help );
		help_menu.add( help_on_typist );
	}
	public void register_about( AboutGUI about ) {
		this.about = about; garbage_collect();
		help_menu.add( help_about );
	}
	public void register_statistics( StatisticsGUI statistics ) {
		this.statistics = statistics; garbage_collect();
		window_menu.add( window_statistics );
	}
	public void register_keyboard( KeyboardGUI keyboard ) {
		this.keyboard = keyboard; garbage_collect();
		window_menu.add( window_keyboard );
	}

	/**
	 * Class test code.
	 */
	public static void main( String args[] ) {
		System.out.println( "*** CLASS TEST CODE ***" );

		ControlGUI control = new ControlGUI( false );
		control.add_series( "N" );
		control.add_series( "S" );

		TutorialGUI tutorial = new TutorialGUI( false );
		tutorial.pack(); tutorial.setVisible( false );

		KBLayout kblayout = KBLayout.create_kblayout( "US" );
		KBStatistics kbstats = new KBStatistics( kblayout );
		StatisticsGUI statistics = new StatisticsGUI( false, kbstats );
		statistics.pack(); statistics.setVisible( false );

		KeyboardGUI keyboard = new KeyboardGUI( false, kblayout );
		keyboard.pack(); keyboard.setVisible( false );

		ExerciseGUI exercise = new ExerciseGUI( false, kbstats );
		exercise.pack(); exercise.setVisible( false );

		HelpGUI help = new HelpGUI( false );
		help.pack(); help.setVisible( false );

		AboutGUI about = new AboutGUI( false, control, false );
		about.pack(); about.setVisible( false );

		exercise.register_statistics( statistics );
		exercise.register_keyboard( keyboard );
		control.register_about( about );
		control.register_tutorial( tutorial );
		control.register_exercise( exercise );
		control.register_statistics( statistics );
		control.register_keyboard( keyboard );
		control.register_help( help );
		control.pack(); control.setVisible( true );
	}
}
