//
// JTypist v3.1 - Java typing tutor program for UNIX systems
// Copyright (C) 2000  Simon Baldwin (simonb@sco.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//
// HelpHTML class - glom of stuff to print online help as HTML
//
import java.util.Vector;
import java.util.StringTokenizer;
import java.util.ResourceBundle;
import java.util.MissingResourceException;
import java.text.MessageFormat;

/**
 * Glom of stuff to print online help as HTML.
 */
abstract public class HelpHTML {

	// message and colour catalog information
	private static final ResourceBundle	messages	=
				ResourceBundle.getBundle( "MessageResource" );
	private static final ResourceBundle	colours		=
				ResourceBundle.getBundle( "ColorResource" );

	// string full of GNU GPL header
	private static final String GNUGPL =
  "<!--\n"
+ "  -- JTypist v3.1 - Java typing tutor program for UNIX system\n"
+ "  -- Copyright (C) 2000  Simon Baldwin (simonb@sco.com)\n"
+ "  --\n"
+ "  -- This program is free software; you can redistribute it and/or\n"
+ "  -- modify it under the terms of the GNU General Public License\n"
+ "  -- as published by the Free Software Foundation; either version 2\n"
+ "  -- of the License, or (at your option) any later version.\n"
+ "  --\n"
+ "  -- This program is distributed in the hope that it will be useful,\n"
+ "  -- but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
+ "  -- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
+ "  -- GNU General Public License for more details.\n"
+ "  --\n"
+ "  -- You should have received a copy of the GNU General Public License\n"
+ "  -- along with this program; if not, write to the Free Software\n"
+ "  -- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.\n"
+ "  --\n"
+ "  --  -->\n";


	/*
	 * Convenience functions to find topic_name and topic_text
	 * from a help topic string.
	 */
	private static String get_topic_name( String s ) {
		return ( s.substring( 0, s.indexOf( "\n" )));
	}
	private static String get_topic_text( String s ) {
		return ( s.substring( s.indexOf( "\n" ) + 1, s.length() ));
	}


	/**
	 * Print out a help text vector as an HTML document.
	 */
	public static String toHTML( Vector helpvector ) {
		StringBuffer buffer = new StringBuffer();

		// write out the HTML header
		buffer.append( "<!DOCTYPE "
				+ messages.getString( "HelpHTML.doctype" )
				+ ">\n" );
		buffer.append( "<HTML>\n" );
		buffer.append( "<HEAD>\n" );
		buffer.append( "  <META HTTP-EQUIV=\"Content-Type\" "
				+ "CONTENT=\""
				+ messages.getString( "HelpHTML.contenttype" )
				+ "\">\n" );
		buffer.append( "  <META NAME=\"GENERATOR\" "
				+ "CONTENT=\"Jtypist v3.1\">\n" );
		buffer.append( "  <TITLE>"
				+ messages.getString( "HelpHTML.title" )
				+ "</TITLE>\n" );
		buffer.append( "</HEAD>\n<BODY TEXT=\""
				+ messages.getString( "HelpHTML.text" )
				+ "\" BGCOLOR=\""
				+ messages.getString( "HelpHTML.bgcolor" )
				+ "\" LINK=\""
				+ messages.getString( "HelpHTML.link" )
				+ "\" VLINK=\""
				+ messages.getString( "HelpHTML.vlink" )
				+ "\" ALINK=\""
				+ messages.getString( "HelpHTML.alink" )
				+ "\">\n" );
		buffer.append( GNUGPL );
		buffer.append( "<H1>"
				+ messages.getString( "HelpHTML.heading" )
				+ "</H1>\n" );
		buffer.append( "<P>\n\n" );

		// write out a contents section
		buffer.append( "<H2>"
				+ messages.getString( "HelpHTML.toc" )
				+ "</H2>\n" );
		buffer.append( "<P>\n\n" );
		buffer.append( messages.getString( "HelpHTML.contains" )
				+ "\n" );
		buffer.append( "<OL>\n" );
		for ( int index = 0; index < helpvector.size(); index++ )
			buffer.append( "<LI><A HREF=\"#topic_" + index + "\">"
				+ get_topic_name(
					(String)(helpvector.elementAt( index )))
					+ "</A></LI>\n" );
		buffer.append( "</OL>\n<P>\n\n" );

		// write out each topic from the help vector
		for ( int index = 0; index < helpvector.size(); index++ ) {
			String helpstring = (String)
						(helpvector.elementAt( index ));
			buffer.append( "<H2><A NAME=\"topic_" + index + "\">"
					+ get_topic_name( helpstring )
					+ "</A></H2>\n" );
			buffer.append( "<P>\n\n" );
			buffer.append( get_topic_text( helpstring ));
			buffer.append( "<P>\n\n" );
		}

		// finish up
		buffer.append( "</BODY>\n</HTML>\n" );
		return ( buffer.toString() );
	}


	/**
	 * Print out the help text as an HTML document.
	 */
	public static String get_HTML_helptext() {

		// find the help text resource for our locale
		ResourceBundle helptext	= null;
		try {
			helptext = ResourceBundle.getBundle(
							"HelpTextResource" );
		}
		catch ( MissingResourceException e ) {}
		catch ( ClassCastException e ) {}
		if ( helptext == null ) {
			System.err.println( messages.getString(
						"HelpGUI.missingresource" ));
			return ( null );
		}

		// find the index of help topics
		String topic_index	= null;
		try {
			topic_index = helptext.getString( "topic_index" );
		}
		catch ( MissingResourceException e ) {
			System.err.println( messages.getString(
						"HelpGUI.missingindex" ));
			return ( null );
		}

		// for each topic in the index, get it and store it in
		// a local vector
		StringTokenizer topics	= new StringTokenizer( topic_index );
		Vector helpvector	 = new Vector( topics.countTokens() );
		while ( topics.hasMoreTokens() ) {
			String topic_ref	= topics.nextToken();
			try {
				helpvector.addElement(
					helptext.getString( topic_ref ));
			}
			catch ( MissingResourceException e ) {
				String[] fmtargs = { topic_ref };
				System.err.println( MessageFormat.format(
					messages.getString(
						"HelpHTML.missingtopic" ),
								fmtargs ));
			}
		}

		// return the help text data, as an HTML string
		return ( toHTML( helpvector ));
	}

	/**
	 * Prints out the HelpTextResource data as HTML.
	 */
	public static void main( String args[] ) {
		System.out.println( get_HTML_helptext() );
	}
}
