//
// JTypist v3.1 - Java typing tutor program for UNIX systems
// Copyright (C) 2000  Simon Baldwin (simonb@sco.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//
// KeyShiftState class - encapsulation of the notion of a key Shift state
//
import java.util.Hashtable;

/**
 * KeyShiftState class gives an enumerated typedef of key Shift states.
 */
public class KeyShiftState {

	// only available definitions
	public final static KeyShiftState	UNSHIFTED
				= new KeyShiftState( "UNSHIFTED" );
	public final static KeyShiftState	SHIFTED
				= new KeyShiftState( "SHIFTED" );
	public final static KeyShiftState	META_ALT
				= new KeyShiftState( "META_ALT" );

	// a table to map state names to values in properties-based layouts
	private static Hashtable	states_by_name = new Hashtable();

	// a static block to populate the name table
	static {
		states_by_name.put( "UNSHIFTED", UNSHIFTED );
		states_by_name.put( "SHIFTED", SHIFTED );
		states_by_name.put( "META_ALT", META_ALT );
	}

	// string id representing the shift state
	private String id;//	= null;

	/**
	 * Create a new KeyShiftState.
	 */
	private KeyShiftState( String id ) {
		this.id = id;
	}

	/**
	 * Find a shiftstate from its logical name.
	 */
	public static KeyShiftState find_keyshiftstate( String name ) {
		return ( (KeyShiftState)(states_by_name.get( name )));
	}

	/**
	 * Return a string representation of the KeyShiftState.
	 */
	public String toString()	{ return ( id ); }
}
